@@ -340,9 +340,9 @@ class Configuration (_Configuration):
 
 	def _clear (self):
 		self.processes = {}
+		self._previous_neighbors = self.neighbors
 		self.neighbors = {}
 		self._neighbors = {}
-		self._previous_neighbors = {}
 
 	# clear the parser data (ie: free memory)
 	def _cleanup (self):
@@ -376,6 +376,7 @@ class Configuration (_Configuration):
 
 	def _rollback_reload (self):
 		self.neighbors = self._previous_neighbors
+		self.processes = self.process.processes
 		self._neighbors = {}
 		self._previous_neighbors = {}
 
@@ -386,10 +387,10 @@ class Configuration (_Configuration):
 		self.processes = self.process.processes
 		self._neighbors = {}
 
-		# installing in the neighbor the API routes
+		# Add the changes prior to the reload to the neighbor to correct handling of deleted routes
 		for neighbor in self.neighbors:
 			if neighbor in self._previous_neighbors:
-				self.neighbors[neighbor].changes = self._previous_neighbors[neighbor].changes
+				self.neighbors[neighbor].backup_changes = self._previous_neighbors[neighbor].changes
 
 		self._previous_neighbors = {}
 		self._cleanup()
@@ -431,6 +432,7 @@ class Configuration (_Configuration):
 
 		if self.section('root') is not True:
 			# XXX: Should it be in neighbor ?
+			self.process.add_api()
 			self._rollback_reload()
 
 			return self.error.set(
